﻿namespace ElmahR
{
    // PostError is a handler which expects a base64-encoded
    // json representation of an error. 

    #region Imports

    using System;
    using System.Web;
    using System.Text;
    using Core;
    using System.Collections.Generic;
    using System.Linq;

    #endregion

    public class PostError : IHttpHandler
    {
        public void ProcessRequest(HttpContext context)
        {
            var @params         = context.Request.Params;

            var error           = Decode(@params["error"]);
            var errorId         = @params["errorId"];
            var sourceId        = @params["sourceId"];

            var infoUrlPath     = string.IsNullOrWhiteSpace(@params["infoUrl"]) 
                                ? "" 
                                : new Uri(@params["infoUrl"]).AbsoluteUri;

            var payload = Error.ProcessAndAppendError(
                sourceId,
                errorId,
                error,
                infoUrlPath,
                ex =>
                {
                    Hub.Log(string.Format("An error occurred in ElmahR: {0}", ex), Hub.Severity.Critical);
                                    
                    Error.ProcessError(
                        Error.ElmahRSourceId,
                        ex,
                        infoUrlPath,
                        _ => { },
                        true);
                },
                true);

            Hub.Log(string.Format("Received error from {0}: {1}", 
                payload.GetApplication().ApplicationName, 
                payload.Message));
        }

        public bool IsReusable
        {
            get
            {
                return false;
            }
        }

        static string Decode(string str)
        {
            return Encoding.UTF8.GetString(Convert.FromBase64String(str));
        }
    }
}